/// <reference types="node" />
import { Logger } from "oly-core";
/**
 *
 */
export declare class FileService {
    protected logger: Logger;
    /**
     * Check if file exists.
     *
     * @param filepath    Absolute path
     */
    exists(filepath: string): Promise<boolean>;
    /**
     * Read a file.
     *
     * @param filepath     Absolute path
     * @param options     Encoding and flag
     */
    read(filepath: string, options: {
        encoding: "UTF-8";
        flag?: string;
    }): Promise<string>;
    /**
     * Write a file.
     *
     * @param filepath   Absolute path
     * @param data      Data
     * @param options   Encode, mode and flag
     */
    write(filepath: string, data: string | Buffer, options?: {
        encoding?: string;
        mode?: number;
        flag?: string;
    }): Promise<void>;
    /**
     * Move a file.
     *
     * @param filepath       Source
     * @param destination   Destination
     */
    move(filepath: string, destination: string): Promise<void>;
    /**
     * Copy a file.
     *
     * @param filepath       Source
     * @param destination   Destination
     */
    copy(filepath: string, destination: string): Promise<void>;
    /**
     * Remove directory/file.
     *
     * @param filepath    Absolute path
     */
    remove(filepath: string): Promise<void>;
    /**
     * Ensure directory.
     *
     * @param filepath    Absolute path
     */
    mkdirp(filepath: string): Promise<void>;
}
